

//% color="#2BA246" iconWidth=40 iconHeight=40
namespace bemfa {
    //% block="tcp server uid:[UID] address:[SERVER_ADDR] server port:[SERVER_PORT]" blockType="command"
    //% SERVER_ADDR.shadow="string" SERVER_ADDR.defl="bemfa.com"
    //% SERVER_PORT.shadow="string" SERVER_PORT.defl="8344"
    //% UID.shadow="string" UID.defl="Uid"
    export function tcp_config(parameter: any, block: any) {
        Generator.addInclude('wifi',`
#include <WiFi.h>
#define TCP_SERVER_ADDR "bemfa.com"
#define TCP_SERVER_PORT "8344"`);

        Generator.addInclude('tcp_config','#include<tcp_config.h>');
        Generator.addInclude('ifndef',`#ifndef AP_CONFIG\nString UID = ${parameter.UID.code};\n#endif`);
        Generator.addCode(`doWiFiTick();`);

        Generator.addInclude('send_to_server',`void sendtoTCPServer(String p){
            if (!TCPclient.connected()){
                Serial.println("Client is not readly");
                return;
            }
            TCPclient.print(p);
        }`);

        Generator.addInclude('sub_topic',`void subTopic(String topic){
            String tcpTemp="";
            tcpTemp = "cmd=1&uid="+UID+"&topic="+topic+"\\r\\n";
            sendtoTCPServer(tcpTemp);
            tcpTemp="";
            Serial.print("sub topic:");
            Serial.println(topic);
        }`);

        Generator.addInclude(`start_tcp`,`void startTCPClient(){
            if(TCPclient.connect(TCP_SERVER_ADDR, atoi(TCP_SERVER_PORT))){
                Serial.print("Connected to server:");
                Serial.println(TCP_SERVER_ADDR);
                subFlag = true;
                preHeartTick = millis();
            TCPclient.setNoDelay(true);
            }else{
                Serial.print("Failed connected to server:");
                Serial.println(TCP_SERVER_ADDR);
                TCPclient.stop();
                preTCPConnected = false;
            }
            preTCPStartTick = millis();
        }`);

        Generator.addInclude(`dowifitick`,`void doWiFiTick()
        {
            static bool taskStarted = false;
            static uint32_t lastWiFiCheckTick = 0;
            if ( WiFi.status() != WL_CONNECTED ) {
                if (millis() - lastWiFiCheckTick > 1000) {
                    lastWiFiCheckTick = millis();
                    WiFi.begin();
                }   
            }else{
                if (taskStarted == false) {
                    taskStarted = true;
                    Serial.print("Get IP Address: ");
                    Serial.println(WiFi.localIP());
                    startTCPClient();
                    } 
            }
        }`);
    }

    //% block="register subscribing topic" blockType="command"
    export function reg_sub(parameter: any,block: any){
        Generator.addCode(`doTCPClientTick();
  subTopicCenter();`);
    }


    //% block="please input your subcribing topic name[TOPIC]" blockType="hat"
    //% TOPIC.shadow="string" TOPIC.defl="Topic"
    export function sub_topic(parameter: any, block: any) {
        Generator.addEvent("callback","void","msg_of_topic_callback","String topic,String msg",true);

        Generator.addInclude(`center`,`void subTopicCenter(){
            if(subFlag == true){
             subFlag = false;
           
               subTopic(${parameter.TOPIC.code});
           
            }
        }`);
        

        
        Generator.addInclude(`doTCPClientTick`,`  void doTCPClientTick(){
            if(WiFi.status() != WL_CONNECTED) return;
            if (!TCPclient.connected()) {
            if(preTCPConnected == true){
              preTCPConnected = false;
             preTCPStartTick = millis();
             Serial.println();
             Serial.println("TCP Client disconnected.");
            TCPclient.stop();
           }else if(millis() - preTCPStartTick > 1*1000)
            startTCPClient();
          }else{
           if (TCPclient.available()) {
             char c =TCPclient.read();
             TcpClient_Buff +=c;
            TcpClient_BuffIndex++;
            TcpClient_preTick = millis();
           if(TcpClient_BuffIndex>=MAX_PACKETSIZE - 1){
             TcpClient_BuffIndex = MAX_PACKETSIZE-2;
             TcpClient_preTick = TcpClient_preTick - 200;
           }
            preHeartTick = millis();
          }
          if(millis() - preHeartTick >= KEEPALIVEATIME){
            preHeartTick = millis();
            Serial.println("--Keep alive:");
            sendtoTCPServer("ping\\r\\n");
           }
           }
           if((TcpClient_Buff.length() >= 1) && (millis() - TcpClient_preTick>=200))
           {
           TCPclient.flush();
            TcpClient_Buff.trim();
             String topic = "";
             String msg = "";
          if(TcpClient_Buff.length() > 15){
            int topicIndex = TcpClient_Buff.indexOf("&topic=")+7;
            int msgIndex = TcpClient_Buff.indexOf("&msg=");
            topic = TcpClient_Buff.substring(topicIndex,msgIndex);
            msg = TcpClient_Buff.substring(msgIndex+5);
          }
        
          msg_of_topic_callback(topic,msg);          

           msg = "";
           topic = "";
          TcpClient_Buff="";
          TcpClient_BuffIndex = 0;
          }
          }`);        
    }


    //% block="msg_of_retn" blockType="reporter"
    export function msg(parameter:any, block:any){
        Generator.addCode('msg');
    }
    
    //% block="topic_name" blockType="reporter"
    export function topic_name(parameter:any, block:any)
    {
        Generator.addCode('topic');

    }    


    //% block="publish MSG  topic:[TOPIC] msg:[MSG]" blockType="command"
    //% TOPIC.shadow="string" TOPIC.defl="Topic"
    //% MSG.shadow="string" MSG.defl="Msg"
    export function publish_msg(parameter: any, block: any) {
        Generator.addInclude(`publish_msg`,`void publishMsg(String topic,String msg){
            String tcpTemp="";
             tcpTemp = "cmd=2&uid="+UID+"&topic="+topic+"&msg="+msg+"\\r\\n";
             sendtoTCPServer(tcpTemp);
             tcpTemp="";
             Serial.println("A message has been published");
             }`);
        Generator.addCode(`publishMsg(${parameter.TOPIC.code}, ${parameter.MSG.code});`);
    }


    //% block="wechat_remind uid[UID] device name:[DEVNAM] msg:[MSG]" blockType="command"
    //% UID.shadow="string" UID.defl="Uid"
    //% DEVNAM.shadow="string" DEVNAM.defl="Device Name"
    //% MSG.shadow="string" MSG.defl="Msg"
    export function wechat_remind(parameter: any, block: any) {
        Generator.addInclude('wechat','#include<wechat.h>');
        let uid=parameter.UID.code;
        let device=parameter.DEVNAM.code;
        let msg=parameter.MSG.code;
        Generator.addCode(`wechat_remind(${uid},${device},${msg});`);
    }



    //% block="wechat_warning uid[UID] device name:[DEVNAM] msg:[MSG]" blockType="command"
    //% UID.shadow="string" UID.defl="Uid"
    //% DEVNAM.shadow="string" DEVNAM.defl="Device Name"
    //% MSG.shadow="string" MSG.defl="Msg"
    export function wechat_warning(parameter: any, block: any) {
        Generator.addInclude('wechat','#include<wechat.h>');
        let uid=parameter.UID.code;
        let device=parameter.DEVNAM.code;
        let msg=parameter.MSG.code;
        Generator.addCode(`wechat_warning(${uid},${device},${msg});`);
    }

}
